/*
 * *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */

package org.nuiton.eugene.models.object;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.commons.digester.Digester;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.ModelReader;
import org.nuiton.eugene.models.object.ObjectModel;
import org.nuiton.eugene.models.object.ObjectModelClass;
import org.nuiton.eugene.models.object.ObjectModelClassifier;
import org.nuiton.eugene.models.object.ObjectModelElement;
import org.nuiton.eugene.models.object.xml.DigesterObjectModelRuleSet;
import org.nuiton.eugene.models.object.xml.ObjectModelClassifierImpl;
import org.nuiton.eugene.models.object.xml.ObjectModelElementImpl;
import org.nuiton.eugene.models.object.xml.ObjectModelImpl;
import org.nuiton.eugene.models.object.xml.ObjectModelImplRef;
import org.nuiton.eugene.models.object.xml.ObjectModelImplTagValue;
import org.nuiton.util.FileUtil;
import org.nuiton.util.RecursiveProperties;
import org.nuiton.util.StringUtil;
import org.xml.sax.SAXException;

/**
 * To read object model files into an memory object model.
 *
 * Created: 26 oct. 2009
 *
 * @author fdesbois
 * @version $Revision: 777 $
 *
 * Mise a jour: $Date: 2009-12-20 17:27:20 +0100 (dim., 20 déc. 2009) $
 * par : $Author: tchemit $
 *
 * @plexus.component role="org.nuiton.eugene.ModelReader" role-hint="objectmodel"
 */
public class ObjectModelReader extends ModelReader<ObjectModel> {

    private static final Log log = LogFactory.getLog(ObjectModelReader.class);
    
    /**
     * L'expression réguliere match les chaines de type
     * <package.ClassName>.<class|attribute|operation>.[name].<stereotype|tagvalue>.[tag]
     * fr.isisfish.entities.Population.class.stereotype=entity
     * fr.isisfish.entities.Population.class.tagvalue.persistenceType=flatfile
     * fr.isisfish.entities.Population.attribute.name.stereotype=...
     * fr.isisfish.entities.Population.attribute.name.tagvalue.pk=topiaId
     * fr.isisfish.entities.Population.operation.getRegion.stereotype=...
     * fr.isisfish.entities.Population.operation.getRegion.tagvalue.pk=...
     */
    protected Pattern propertiesPattern = Pattern
            .compile("((?:[_a-zA-Z0-9]+\\.)+(?:_?[A-Z][_a-zA-Z0-9]*\\.)+)(?:(class|attribute|operation)\\.)(?:([_a-z0-9][_a-zA-Z0-9]*)\\.)?(?:(stereotype|tagvalue)\\.?)([_a-z0-9][_a-zA-Z0-9]*)?");


    /**
     * @param files les noms des fichiers existant contenant du XML représentant
     *        des ObjectModel. Il est automatiquement recherche un fichier de
     *        propriété associé a ce fichier pour pouvoir ajouter des
     *        stereotype ou des tag value sur les class, attribute ou operation
     */
    @Override
    public ObjectModel read(File[] files) {
        setLastModifiedSource(files);
        ObjectModel objectModel = new ObjectModelImpl();

        // Digester parser configuration
        Digester digester = new Digester();
        digester.addRuleSet(new DigesterObjectModelRuleSet());

        for (File file : files) {
            try {
                digester.push(objectModel);
                objectModel = (ObjectModel) digester.parse(file);
            } catch (IOException e) {
                log.warn("Unable to parse ObjectModel input file : " + file, e);
            } catch (SAXException e) {
                log.warn("Unable to parse ObjectModel input file : " + file, e);
            }

            // recherche est charge le fichier propriete associe au modele
            File dir = file.getParentFile();
            String ext = FileUtil.extension(file);
            String name = FileUtil.basename(file, "." + ext);
            File propFile = new File(dir, name + ".properties");
            RecursiveProperties prop = new RecursiveProperties();
            if (!propFile.exists()) {
                if (log.isInfoEnabled()) {
                    log.info("Pas de fichier de propriete " + propFile
                            + " associé au model");
                }
            } else {
                if (log.isInfoEnabled()) {
                    log.info("Lecture du fichier de propriete " + propFile
                            + " associé au model");
                }
                try {
                    prop.load(new FileInputStream(propFile));
                } catch (IOException eee) {
                    log.warn("Impossible de lire le fichier de propriete "
                            + propFile, eee);
                }

                // on ajoute les proprietes du fichier associe au model
                for (Enumeration<Object> e = prop.keys(); e.hasMoreElements();) {
                    String key = (String) e.nextElement();
                    String value = prop.getProperty(key);

                    if (key.startsWith("model")) {
                        if (!key.startsWith("model.tagvalue.")) {
                            log.warn("only tagvalue is allowed on model in properties");
                        } else {
                            String tag = key.substring("model.tagvalue."
                                    .length());

                            ObjectModelImplTagValue tagvalue = new ObjectModelImplTagValue();
                            tagvalue.setName(tag);
                            tagvalue.setValue(value);
                            ((ObjectModelImpl) objectModel)
                                    .addTagValue(tagvalue);
                        }
                    } else {

                        Matcher matcher = propertiesPattern.matcher(key);

                        if (log.isDebugEnabled()) {
                            log.debug("Propriete: '" + key + "'");
                        }

                        if (matcher.find()) {
                            // fqn is fully qualified name of class
                            String fqn = matcher.group(1);
                            fqn = StringUtil.substring(fqn, 0, -1); // remove ended
                            // .
                            // target is class, attribute or operation
                            String target = matcher.group(2);
                            String targetName = matcher.group(3);
                            // type is stereotype or tagvalue
                            String type = matcher.group(4);
                            String tag = matcher.group(5);

                            if (log.isDebugEnabled()) {
                                log.debug("Propriete: '" + key + "' => "
                                        + "fqn:" + fqn + " target:" + target
                                        + " targetName:" + targetName
                                        + " type:" + type + " tag:" + tag);
                            }

                            ObjectModelClassifier omc = objectModel
                                    .getClassifier(fqn);
                            if (omc instanceof ObjectModelClassifierImpl) {
                                // on utilise une liste car il peut y avoir plusieur
                                // operation avec le nom donné
                                List<ObjectModelElement> elems = new ArrayList<ObjectModelElement>();
                                if ("class".equals(target)) {
                                    elems.add(omc);
                                } else if ("attribute".equals(target)) {
                                    ObjectModelClass classmodel = (ObjectModelClass) omc;
                                    elems.add(classmodel
                                            .getAttribute(targetName));
                                } else if ("operation".equals(target)) {
                                    elems.addAll(omc.getOperations(targetName));
                                }
                                for (Object elem1 : elems) {
                                    ObjectModelElementImpl elem = (ObjectModelElementImpl) elem1;
                                    if ("stereotype".equals(type)) {
                                        // pour les stereotypes
                                        ObjectModelImplRef stereotype = new ObjectModelImplRef();
                                        stereotype.setName(value);
                                        elem.addStereotype(stereotype);
                                        continue;
                                    }

                                    if ("tagvalue".equals(type)) {
                                        // pour les tagvalues
                                        ObjectModelImplTagValue tagvalue = new ObjectModelImplTagValue();
                                        tagvalue.setName(tag);
                                        tagvalue.setValue(value);
                                        if (log.isDebugEnabled()) {
                                            log.debug("tagValue:" + tagvalue
                                                    + ", tag:" + tag
                                                    + ", value:" + value
                                                    + ", element:" + elem);
                                        }
                                        if (elem == null) {
                                            log.warn("can not attach tagValue "
                                                    + key);
                                        } else {
                                            elem.addTagValue(tagvalue);
                                        }
                                    }
                                }
                            } else {
                                // TODO il faudra avoir des methodes d'acces en
                                // Set sur l'interface pour eviter ce message
                                if (log.isWarnEnabled()) {
                                    log.warn("Can't add properties to model, it's not an ObjectModelClassifierImpl");
                                }
                            }
                        }
                    }
                }
            }
        }
        if (log.isDebugEnabled()) {
            for (ObjectModelClass m : objectModel.getClasses()) {
                log.debug("loaded class in objectmodel : " + m.getName());
            }
        }
        return objectModel;
        /*try {
            generate(objectModel, destDir);
        } catch (IOException eee) {
            if (log.isWarnEnabled()) {
                log.warn("Unable to generate for file", eee);
            }
        }*/
    }
}
