/*
 * \*##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */

package org.nuiton.eugene.models.object.xml;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.nuiton.eugene.models.object.ObjectModelElement;

/**
 * ObjectModelElementImpl.
 *
 * @author chatellier
 * @author cedric
 * @version $Revision: 777 $
 *
 * Last update : $Date: 2009-12-20 17:27:20 +0100 (dim., 20 déc. 2009) $
 * By : $Author: tchemit $
 */
public class ObjectModelElementImpl implements ObjectModelElement {

    protected ObjectModelImpl objectModelImpl;
    protected ObjectModelElement declaringElement;

    protected String name = null;
    protected boolean isStatic = false;
    protected String documentation = null;
    protected List<String> stereotypes = new ArrayList<String>();
    protected Map<String, String> tagValues = new HashMap<String, String>();
    protected List<String> comments = new ArrayList<String>();

    public ObjectModelElementImpl() {
        super();
    }

    /**
     * @param objectModelImpl the objectModelImpl
     */
    public void setObjectModelImpl(ObjectModelImpl objectModelImpl) {
        this.objectModelImpl = objectModelImpl;
    }

    /**
     * @param declaringElement the declaringElement to set
     */
    public void setDeclaringElement(ObjectModelElement declaringElement) {
        this.declaringElement = declaringElement;
    }

    /**
     * TODO a tester
     * @return the objectModel
     */
    public ObjectModelImpl getModel() {
        if (objectModelImpl != null) {
            return objectModelImpl;
        } else if (declaringElement != null) {
            return ((ObjectModelElementImpl) declaringElement).getModel();
        }
        return null;
    }

    public void postInit() {
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setDocumentation(String documentation) {
        this.documentation = documentation;
    }

    public ObjectModelImplRef addStereotype(ObjectModelImplRef stereotype) {
        if (stereotype == null) {
            return new ObjectModelImplRef();
        }
        stereotypes.add(stereotype.getName());
        return stereotype;
    }

    public ObjectModelImplTagValue addTagValue(ObjectModelImplTagValue tagValue) {
        if (tagValue == null) {
            return new ObjectModelImplTagValue();
        }
        tagValues.put(tagValue.getName(), tagValue.getValue());
        return tagValue;
    }

    public void setStatic(boolean isStatic) {
        this.isStatic = isStatic;
    }

    public void addComment(String comment) {
        this.comments.add(comment);
    }

    @Override
    public String getName() {
        return name;
    }

    /**
    * Returns the element in which this element is defined, or null if there's none.
    * 
    * @return  the ObjectModelElement in which this element is defined, or null if there's none.
    */
    @Override
    public ObjectModelElement getDeclaringElement() {
        return declaringElement;
    }

    /**
    * Returns the whole documentation associated with this element (description + source documentation).
    * 
    * @return the whole documentation associated with this element.
    */
    @Override
    public String getDocumentation() {
    	if (documentation == null && hasTagValue("documentation")) {
    		return getTagValue("documentation");
    	}
        return documentation;
    }

    /**
    * Returns the description associated with this element.
    * 
    * @return the description associated with this element.
    */
    @Override
    public String getDescription() {
        return getDocumentation().substring(0, getDocumentation().indexOf("--"));
    }

    /**
    * Returns the source documentation part associated with this element.
    * 
    * @return the source documentation part associated with this element.
    */
    @Override
    public String getSourceDocumentation() {
        return getDocumentation().substring(getDocumentation().indexOf("--") + 2);
    }

    /**
    * Returns the stereotypes names associated with this element.
    * 
    * @return a Collection containing all stereotypes names associated with this element as String.
    */
    @Override
    public Collection<String> getStereotypes() {
        return stereotypes;
    }

    /**
    * Returns whether this element has a stereotype corresponding to the given name, or not.
    * 
    * @return a boolean indicating whether this element has a stereotype corresponding to the given name, or not.
    */
    @Override
    public boolean hasStereotype(String stereotypeName) {
        return stereotypes.contains(stereotypeName);
    }

    /**
    * Returns the tagValues associated with this element.
    * For each entry, the key is the name of the tagValue, the value is the value of the tagValue :-) 
    * 
    * @return a Map containing all tagValues associated with this element
    */
    @Override
    public Map<String, String> getTagValues() {
        return tagValues;
    }

    /**
    * Returns the tagValue corresponding to the given name, or null if the element has no associated tagValue for this name.
    * 
    * @return the value of the found tagValue, or null if the element has no associated tagValue for this name.
    */
    @Override
    public String getTagValue(String tagValue) {
        return (tagValue == null ? null : tagValues.get(tagValue));
    }

    /**
    * Returns whether this element has a tagValue corresponding to the given name, or not.
    * 
    * @return a boolean indicating whether this element has a tagValue corresponding to the given name, or not.
    */
    @Override
    public boolean hasTagValue(String tagValue) {
        return tagValues.containsKey(tagValue);
    }

    @Override
    public boolean isStatic() {
        return isStatic;
    }

    /**
    * Returns all comments lied to this particular model element
    * 
    * @return a List containing all comments for this element as Strings.
    */
    @Override
    public List<String> getComments() {
        return comments;
    }

}
