/* *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*/

package org.nuiton.eugene.models.state;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.AbstractGenerator;
import org.nuiton.eugene.MonitorWriter;
import org.nuiton.eugene.models.state.StateModel;
import org.nuiton.eugene.models.state.StateModelState;
import org.nuiton.eugene.models.state.StateModelStateChart;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

/**
 * StateModelGenerator.
 * <p/>
 * Pour utiliser ce type de générateur, il faut implanter au moins une des
 * méthodes generateFrom... et le getFilenameFor... associé si l'on souhaite un
 * nom de fichier convenable. Si dans une méthode generateFrom... on utilise pas
 * le writer (output) alors aucun fichier n'est généré.
 * <p/>
 * Le nom de l'argument writer doit absolument etre output et pas autre chose si
 * vous souhaitez utiliser le processeur
 * {@code org.nuiton.processor.filters.GeneratorTemplatesFilter} pour vous
 * s'implifier l'écriture des templates.
 *
 * @author chatellier
 * @version $Revision: 777 $
 *          <p/>
 *          Last update : $Date: 2009-12-20 17:27:20 +0100 (dim., 20 déc. 2009) $ By : $Author: tchemit $
 */
public class StateModelGenerator extends AbstractGenerator<StateModel> {

    /**
     * Logger for this class
     */
    private static Log log = LogFactory.getLog(StateModelGenerator.class);

    /**
     * Empty constructor
     */
    public StateModelGenerator() {
        super();
    }

    /**
     * Constructor with parent generator.
     *
     * @param parent parent generator
     */
    public StateModelGenerator(AbstractGenerator<StateModel> parent) {
        super(parent);
    }

    /*
     * @see org.nuiton.eugene.Generator#generate(java.io.File[], java.io.File)
     */
    @Override
    @Deprecated
    public void generate(File[] files, File destDir) {

        StateModelReader reader = new StateModelReader();
        StateModel stateModel = reader.read(files);
        setLastModifiedSource(reader.getLastModifiedSource());

        // generate code
        try {
            applyTemplate(stateModel, destDir);
        } catch (IOException e) {
            log.warn("Can't generate code for files", e);
        }
    }


    /**
     * Par défaut, appel {@link #generateFromModel(org.nuiton.eugene.MonitorWriter , org.nuiton.eugene.models.state.StateModel)} pour le
     * model et {@link #generateFromState(Writer,StateModelState)} pour tous les
     * etats du modele.
     *
     * @param stateModel Le modele d'état
     * @param destDir    le dossier de destination
     * @throws IOException
     */
    @Override
    public void applyTemplate(StateModel stateModel, File destDir) throws IOException {

        model = stateModel;

        String filename = getFilenameFromModel(stateModel);
        File outputFile = getDestinationFile(destDir, filename);
        if (getOverwrite() || !isNewerThanSource(outputFile)) {
            try {
                StringWriter out = new StringWriter();
                MonitorWriter monitorOut = new MonitorWriter(out);
                generateFromModel(monitorOut, stateModel);
                write(outputFile, monitorOut);
            } catch (Exception eee) {
                log.warn("Erreur lors de la génération du fichier "
                        + outputFile);
                throw new RuntimeException(
                        "Erreur lors de la génération du fichier " + outputFile,
                        eee);
            }
        }

        // pour tous les diagrammes du modele
        for (StateModelStateChart chart : stateModel.getStateCharts()) {

            // elements can be restricted in package
            if (canGenerateElement(chart)) {

                // et tous les états de ces diagrammes
                for (Object oState : chart.getStates().toArray()) {
                    StateModelState state = (StateModelState) oState;
                    String filenameState = getFilenameFromState(state, chart
                            .getName());
                    File outputFiletate = getDestinationFile(destDir, filenameState);
                    if (getOverwrite() || !isNewerThanSource(outputFiletate)) {
                        try {
                            StringWriter out = new StringWriter();
                            MonitorWriter monitorOut = new MonitorWriter(out);
                            generateFromState(monitorOut, state);
                            write(outputFiletate, monitorOut);
                        } catch (Exception eee) {
                            log.warn("Erreur lors de la génération du fichier "
                                    + outputFiletate);
                            throw new RuntimeException(
                                    "Erreur lors de la génération du fichier "
                                            + outputFiletate, eee);
                        }
                    }
                }
            }
        }
    }

    /**
     * Test if given element can be generated.
     *
     * @param chart chart to test
     * @return generation allowed
     */
    protected boolean canGenerateElement(StateModelStateChart chart) {

        boolean canGenerate = true;

        // disabled until tested
        //String chartPackage = chart.getPackageName();
        //canGenerate = super.canGeneratePackage(chartPackage);

        return canGenerate;
    }

    /**
     * Generate model code
     *
     * @param monitorOut
     * @param stateModel
     */
    protected void generateFromModel(MonitorWriter monitorOut, StateModel stateModel) {

    }

    /**
     * Return filename from model
     *
     * @param stateModel
     * @return model file name
     */
    protected String getFilenameFromModel(StateModel stateModel) {
        return stateModel.getName();
    }

    /**
     * Return filename from state
     *
     * @param state       the state
     * @param packageName
     * @return the filename
     */
    public String getFilenameFromState(StateModelState state, String packageName) {
        return (packageName + '.' + state.getName()).replace('.', File.separatorChar);
    }

    /**
     * Generate a state code
     *
     * @param monitorOut the output writer
     * @param state      the state
     * @throws IOException
     */
    public void generateFromState(Writer monitorOut, StateModelState state) throws IOException {

    }
}
