package org.nuiton.eugene.writer;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Contract of the chained writer engine.
 * <p/>
 * You can register inputs via the method {@link #registerInclude(String)}.
 * <p/>
 * And later obtain of the selected writers for your inputs via the method
 * {@link #getSelectedWriters()}.
 * <p/>
 * Created: 17 déc. 2009
 *
 * @author Tony Chemit <chemit@codelutin.com> Copyright Code Lutin
 * @version $Revision: 835 $
 *          <p/>
 *          Mise a jour: $Date: 2010-03-04 19:03:17 +0100 (jeu., 04 mars 2010) $
 *          par : $Author: tchemit $
 * @since 2.0.0
 */
public interface ChainedWriterEngine {
    /**
     * Register in engine a new input source.
     * <p/>
     * this method will detects writers to register and chain them if
     * necessary.
     *
     * @param include the new include to digest
     */
    void registerInclude(String include);

    /** @return the common configuration of engine */
    ChainedFileWriterConfiguration getConfiguration();

    /**
     * Sets the common configuration.
     *
     * @param configuration the new configuration ot use.
     */
    void setConfiguration(ChainedFileWriterConfiguration configuration);

    /** @return the set of all available writers discovered at runtime */
    Set<ChainedFileWriter> getAvailableWriters();

    /** @return the list of selected writers after having registred some inputs. */
    List<ChainedFileWriter> getSelectedWriters();

    /**
     * Tests if there is a selected writer using the given input protocol.
     *
     * @param inputProtocol the inputProtocol to test
     * @return {@code true} if there is a selected writer using this input
     *         protocol
     * @see ChainedFileWriter#getInputProtocol()
     * @see #getSelectedWriters()
     */
    boolean containsWriter(String inputProtocol);

    /**
     * Filter the given {@code universe} of writers which accept the given
     * {@code modelType}.
     *
     * @param universe  the list of writers to filter
     * @param modelType the accepted model type
     * @return the set of filtered writers
     * @see ChainedFileWriter#acceptModel(String)
     */
    Set<ChainedFileWriter> filterWriterForModelType(
            Map<String, ChainedFileWriter> universe,
            String modelType);

    /**
     * Filter the given {@code universe} of writers which accept the given
     * {@code modelType} and {@code inputProtocol}.
     *
     * @param universe      the list of writers to filter
     * @param inputProtocol the accepted input protocol
     * @param modelType     the accepted model type
     * @return the set of filtered writers
     * @see ChainedFileWriter#acceptModel(String)
     * @see ChainedFileWriter#getInputProtocol()
     * @see ChainedFileWriter#getInputProtocol(String)
     */
    ChainedFileWriter getWriterForInputProtocol(
            Set<ChainedFileWriter> universe,
            String inputProtocol,
            String modelType);

    /**
     * Filter the given {@code universe} of writers which accept the given
     * {@code modelType} and {@code include}.
     * <p/>
     * The include can have several forms : <ul> <li></li> <li></li> </ul>
     *
     * @param universe  the list of writers to filter
     * @param include   the include configuration
     * @param modelType the accepted model type
     * @return the set of filtered writers
     * @see ChainedFileWriter#acceptModel(String)
     * @see ChainedFileWriter#getInputProtocol()
     * @see ChainedFileWriter#getInputProtocol(String)
     */
    ChainedFileWriter getWriterForInclude(
            Set<ChainedFileWriter> universe,
            String include,
            String modelType);

    /** clean internal states */
    void clear();
}
